import greenfoot.*;

public class Man extends Actor
{
    private GreenfootSound advance = new GreenfootSound("advance.mp3");
    private GreenfootSound eatFruit = new GreenfootSound("fruit.mp3");
    private GreenfootSound deadman = new GreenfootSound("die.mp3");
    private GreenfootSound spirit = new GreenfootSound("spirit.mp3");
    
    private GreenfootImage[] images = new GreenfootImage[4];
    
    private int imageTimer = 16;
    private int lifeVal = 61;
    private int spiritTimer;
    
    public Man()
    {
        for (int i=0; i<4; i++)
        {
            images[i] = new GreenfootImage("pacman"+i+".png");
            images[i].mirrorHorizontally();
        }
        setImage(images[1]);
    }
    
    public void addedToWorld(World w)
    {
        w.removeObjects(getIntersectingObjects(Food.class));
    }

    public void act()
    {
        if (!(((Maze)getWorld()).running)) return;
        if (lifeVal == 0) { dead(); return; }
        if (lifeVal < 61) { die(); return; }
        move();
        checkFood();
        checkPill();
        checkFruit();
        checkSpirit();
        checkGhost();
    }
    
    private void move()
    {
        int dx = 0, dy = 0;
        if (Greenfoot.isKeyDown("right")) dx++;
        if (Greenfoot.isKeyDown("down")) dy++;
        if (Greenfoot.isKeyDown("left")) dx--;
        if (Greenfoot.isKeyDown("up")) dy--;
        if (dx*dy == 0 && dx+dy != 0)
        {
            int direction = getRotation();
            setRotation(90*((1-dx)*(dx*dx)+(2-dy)*(dy*dy)));
            if (!canMove()) setRotation(direction);
        }
        if (canMove())
        {
            move(2);
            imageTimer = (imageTimer+1)%16;
        }
        else imageTimer = 12;
        setImage(images[imageTimer/4]);
        if (getX() < 0) setLocation(getWorld().getWidth(), getY());
        if (getX() > getWorld().getWidth()) setLocation(0, getY());
    }

    private boolean canMove()
    {
        move(2);
        Actor wall = getOneIntersectingObject(Wall.class);
        move(-2);
        return wall == null;
    }
    
    private void checkFood()
    {
        Actor actor = getOneIntersectingObject(Food.class);
        if (actor == null) return;
        advance.stop();
        advance.play();
        getWorld().removeObject(actor);
        ((Maze)getWorld()).scoreCounter.add(10);        
    }
    
    private void checkPill()
    {
        Actor actor = getOneIntersectingObject(Pill.class);
        if (actor == null) return;
        getWorld().removeObject(actor);
        advance.stop();
        advance.play();
        ((Maze)getWorld()).scoreCounter.add(50);
        for (Object obj : getWorld().getObjects(Ghost.class)) ((Ghost)obj).setWeak();
    }
    
    private void checkFruit()
    {
        Actor actor = getOneIntersectingObject(Fruit.class);
        if (actor == null) return;
        eatFruit.play();
        getWorld().removeObject(actor);
        ((Maze)getWorld()).scoreCounter.add(((Fruit)actor).getValue());
    }
    
    private void checkSpirit()
    {
        if (spiritTimer > 0)
        {
            spiritTimer--;
            if (spiritTimer > 0)
            {
                int trans = getImage().getTransparency();
                if (trans > 1) getImage().setTransparency(trans-5);
            }
            else
            {
                for (int i=0; i<4; i++) images[i].setTransparency(255);
            }
            return;
        }
        Actor actor = getOneIntersectingObject(Spirit.class);
        if (actor == null) return;
        spirit.play();
        getWorld().removeObject(actor);
        spiritTimer = 400;
    }
    
    private void checkGhost()
    {
        Actor actor = getOneIntersectingObject(Ghost.class);
        if (actor == null) return;
        Ghost ghost = (Ghost)actor;
        if (ghost.isDead()) return;
        if (ghost.isWeak())
        {
            ghost.die();
            ((Maze)getWorld()).scoreCounter.add(200);
            return;
        }
        deadman.play();
        getWorld().removeObject(ghost);
        lifeVal--;
    }
    
    private void die()
    {
        if (lifeVal%10 == 0)
        {
            setRotation(180);
            setImage(new GreenfootImage("deadman.png"));
            int imgNum = lifeVal/20-1;
            if ((lifeVal/10)%2 == 1) imgNum = 6-(lifeVal/20+1);
            if (imgNum < 5)  getImage().drawImage(new GreenfootImage("pacman"+imgNum+".png"), 0, 0);
        }
        lifeVal--;        
    }
    
    private void dead()
    {
        Maze maze = (Maze)getWorld();
        maze.lives.adjustValue(-1);
        maze.removeObject(this);
        if (maze.lives.getValue() > 0) return;
        if (Maze.highCounter.getValue() < maze.scoreCounter.getValue())
            Maze.highCounter.setValue(maze.scoreCounter.getValue());
        Greenfoot.stop();
    }
}