import greenfoot.*;

public class Maze extends World
{
    public static Counter highCounter;

    private GreenfootSound ready = new GreenfootSound("ready.mp3");
    private GreenfootSound gameplay = new GreenfootSound("gameplay.mp3");
    public Color BLUE = new Color(192, 192, 255);
    
    public boolean running, spirited;
    private int level;
    public Counter scoreCounter;
    public Lives lives;
    public int fruit, bonuses;
    
    public Maze()
    {
        super(456, 576, 1, false);
        GreenfootImage bg = getBackground();
        bg.setColor(new Color(192, 192, 255));
        bg.fill();
        lives = new Lives();
        scoreCounter = new Counter("Score", 0);
        if (highCounter == null) highCounter = new Counter("High score", 0);
        level = 0;
    }
    
    public void reset()
    {
        if (getObjects(Pill.class).isEmpty() && getObjects(Food.class).isEmpty()) newLevel();
        resetMovers();
    }
    
    private void newLevel()
    {
        level++;
        Greenfoot.setSpeed(44+level/2);
        spirited = false;
        removeObjects(getObjects(null));
        running = false;
        addObject(lives, 228, 36);
        addObject(scoreCounter, 114, 12);
        addObject(highCounter, 341, 12);
        for (int i=1; i<39; i+=2) for (int j=1; j<45; j+=2) addObject(new Food(), i*12, 48+j*12);
        int[][] map = (new Map()).getMap(0);
        for (int i=0; i<map.length; i++) for (int j=0; j<map[0].length; j++)
        {
            if (map[i][j] >= 0) addObject(new Wall(map[i][j]), 12+j*24, 60+i*24);
            if (map[i][j] == -2) addObject(new Path(), 12+j*24, 60+i*24);
            if (map[i][j] == -3) addObject(new Pill(), 12+j*24, 60+i*24);
        }
    }
        
    private void resetMovers()
    {
        running = false;
        removeObjects(getObjects(Fruit.class));
        removeObjects(getObjects(Man.class));
        removeObjects(getObjects(Spirit.class));
        addObject(new Man(), 228, 444);
        removeObjects(getObjects(Ghost.class));
        for (int i=0; i<4; i++) addObject(new Ghost(), 204+((i+1)%3)*24, 300);
        ready.play();
    }
    
    public void act()
    {
        if (!running && !ready.isPlaying()) running = true;
        if (!running) { gameplay.stop(); return; }
        if (!gameplay.isPlaying()) gameplay.play();
        if ((bonuses == 0 && scoreCounter.getValue() >= 10000) || (bonuses == 1 && scoreCounter.getValue() >= 20000))
        {
            bonuses++;
            lives.adjustValue(1);
        }
        if ((getObjects(Pill.class).isEmpty() && getObjects(Food.class).isEmpty()) || 
             (getObjects(Man.class).isEmpty() && ((Lives)getObjects(Lives.class).get(0)).getValue() > 0) )
                reset();
        if (getObjects(Man.class).isEmpty())
        {
            lives = null;
            Greenfoot.setWorld(new Maze());
            return;
        }
        if (getObjects(Fruit.class).isEmpty() && Greenfoot.getRandomNumber(3000) == 0)
        {
            int fruitNum = fruit;
            if (fruit == 5) fruitNum = Greenfoot.getRandomNumber(5); else fruit++;
            addObject(new Fruit(fruitNum), 228, 348);
        }
        if (level%2 == 0 && !spirited && ((Man)getObjects(Man.class).get(0)).getY() == 516)
        {
            spirited = true;
            addObject(new Spirit(), 228, 252);
        }
    }
}